@@ -14,10 +14,10 @@
  * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 /*
- * Programmer:	Quincey Koziol <koziol@hdfgroup.org>
+ * Programmer:  Quincey Koziol <koziol@hdfgroup.org>
  *              Monday, July 31, 2006
  *
- * Purpose:	Free space tracking functions.
+ * Purpose:     Free space tracking functions.
  *
  */
 
@@ -25,7 +25,7 @@
 /* Module Setup */
 /****************/
 
-#define H5FS_PACKAGE		/*suppress error about including H5FSpkg  */
+#include "H5FSmodule.h"         /* This source code file is part of the H5FS module */
 
 
 /***********/
@@ -111,14 +111,14 @@ H5FL_DEFINE(H5FS_sinfo_t);
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sinfo_new
+ * Function:    H5FS_sinfo_new
  *
- * Purpose:	Create new section info structure
+ * Purpose:     Create new section info structure
  *
- * Return:	Success:	non-NULL, pointer to new section info struct
- *		Failure:	NULL
+ * Return:      Success:    non-NULL, pointer to new section info struct
+ *              Failure:    NULL
  *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Monday, July 31, 2006
  *
  *-------------------------------------------------------------------------
@@ -126,8 +126,8 @@ H5FL_DEFINE(H5FS_sinfo_t);
 H5FS_sinfo_t *
 H5FS_sinfo_new(H5F_t *f, H5FS_t *fspace)
 {
-    H5FS_sinfo_t *sinfo = NULL; /* Section information struct created */
-    H5FS_sinfo_t *ret_value;    /* Return value */
+    H5FS_sinfo_t *sinfo = NULL;         /* Section information struct created */
+    H5FS_sinfo_t *ret_value = NULL;     /* Return value */
 
     FUNC_ENTER_NOAPI_NOINIT
 
@@ -140,7 +140,7 @@ HDfprintf(stderr, "%s: fspace->addr = %a\n", FUNC, fspace->addr);
 
     /* Allocate the free space header */
     if(NULL == (sinfo = H5FL_CALLOC(H5FS_sinfo_t)))
-	HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
+        HGOTO_ERROR(H5E_RESOURCE, H5E_NOSPACE, NULL, "memory allocation failed")
 
     /* Set non-zero values */
     sinfo->nbins = H5VM_log2_gen(fspace->max_sect_size);
@@ -186,30 +186,30 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sinfo_lock
+ * Function:    H5FS_sinfo_lock
  *
- * Purpose:	Make certain the section info for the free space manager is
+ * Purpose:     Make certain the section info for the free space manager is
  *              in memory.
  *
  *              Either uses existing section info owned by the free space
  *              header, loads section info from disk, or creates new section
  *              info
  *
- * Return:	Success:	non-negative
- *		Failure:	negative
+ * Return:      SUCCEED/FAIL
  *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Thursday, February  7, 2008
  *
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5FS_sinfo_lock(H5F_t *f, hid_t dxpl_id, H5FS_t *fspace, H5AC_protect_t accmode)
+H5FS_sinfo_lock(H5F_t *f, hid_t dxpl_id, H5FS_t *fspace, unsigned accmode)
 {
     H5FS_sinfo_cache_ud_t cache_udata; /* User-data for cache callback */
     herr_t ret_value = SUCCEED;    /* Return value */
 
-    FUNC_ENTER_NOAPI_NOINIT
+    FUNC_ENTER_NOAPI_NOINIT_TAG(dxpl_id, H5AC__FREESPACE_TAG, FAIL)
+
 #ifdef H5FS_SINFO_DEBUG
 HDfprintf(stderr, "%s: Called, fspace->addr = %a, fspace->sinfo = %p, fspace->sect_addr = %a\n", FUNC, fspace->addr, fspace->sinfo, fspace->sect_addr);
 HDfprintf(stderr, "%s: fspace->alloc_sect_size = %Hu, fspace->sect_size = %Hu\n", FUNC, fspace->alloc_sect_size, fspace->sect_size);
@@ -219,14 +219,21 @@ HDfprintf(stderr, "%s: fspace->alloc_sect_size = %Hu, fspace->sect_size = %Hu\n"
     HDassert(f);
     HDassert(fspace);
 
+    /* only H5AC__READ_ONLY_FLAG may appear in accmode */
+    HDassert((accmode & (unsigned)(~H5AC__READ_ONLY_FLAG)) == 0);
+
     /* If the free space header doesn't already "own" the section info, load
      *  section info or create it
      */
     if(fspace->sinfo) {
         /* Check if the section info was protected & we want a different access mode */
+
+        /* only H5AC__READ_ONLY_FLAG may appear in fspace->sinfo_accmode */
+        HDassert(((fspace->sinfo_accmode) & (unsigned)(~H5AC__READ_ONLY_FLAG)) == 0);
+
         if(fspace->sinfo_protected && accmode != fspace->sinfo_accmode) {
             /* Check if we need to switch from read-only access to read-write */
-            if(H5AC_WRITE == accmode) {
+	    if(0 == (accmode & (unsigned)(~H5AC__READ_ONLY_FLAG))) {
                 /* Unprotect the read-only section info */
                 if(H5AC_unprotect(f, dxpl_id, H5AC_FSPACE_SINFO, fspace->sect_addr, fspace->sinfo, H5AC__NO_FLAGS_SET) < 0)
                     HGOTO_ERROR(H5E_FSPACE, H5E_CANTUNPROTECT, FAIL, "unable to release free space section info")
@@ -235,11 +242,11 @@ HDfprintf(stderr, "%s: fspace->alloc_sect_size = %Hu, fspace->sect_size = %Hu\n"
                 cache_udata.f = f;
                 cache_udata.dxpl_id = dxpl_id;
                 cache_udata.fspace = fspace;
-                if(NULL == (fspace->sinfo = (H5FS_sinfo_t *)H5AC_protect(f, dxpl_id, H5AC_FSPACE_SINFO, fspace->sect_addr, &cache_udata, H5AC_WRITE)))
+                if(NULL == (fspace->sinfo = (H5FS_sinfo_t *)H5AC_protect(f, dxpl_id, H5AC_FSPACE_SINFO, fspace->sect_addr, &cache_udata, H5AC__NO_FLAGS_SET)))
                     HGOTO_ERROR(H5E_FSPACE, H5E_CANTPROTECT, FAIL, "unable to load free space sections")
 
                 /* Switch the access mode we have */
-                fspace->sinfo_accmode = H5AC_WRITE;
+                fspace->sinfo_accmode = H5AC__NO_FLAGS_SET;
             } /* end if */
         } /* end if */
     } /* end if */
@@ -291,20 +298,19 @@ done:
 HDfprintf(stderr, "%s: Leaving, fspace->addr = %a, fspace->sinfo = %p, fspace->sect_addr = %a\n", FUNC, fspace->addr, fspace->sinfo, fspace->sect_addr);
 HDfprintf(stderr, "%s: fspace->alloc_sect_size = %Hu, fspace->sect_size = %Hu\n", FUNC, fspace->alloc_sect_size, fspace->sect_size);
 #endif /* H5FS_SINFO_DEBUG */
-    FUNC_LEAVE_NOAPI(ret_value)
+    FUNC_LEAVE_NOAPI_TAG(ret_value, FAIL)
 } /* H5FS_sinfo_lock() */
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sinfo_unlock
+ * Function:    H5FS_sinfo_unlock
  *
- * Purpose:	Release the section info, either giving ownership back to
+ * Purpose:     Release the section info, either giving ownership back to
  *              the cache or letting the free space header keep it.
  *
- * Return:	Success:	non-negative
- *		Failure:	negative
+ * Return:      SUCCEED/FAIL
  *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Thursday, February  7, 2008
  *
  *-------------------------------------------------------------------------
@@ -330,7 +336,7 @@ HDfprintf(stderr, "%s: fspace->alloc_sect_size = %Hu, fspace->sect_size = %Hu\n"
     /* Check if we modified any section */
     if(modified) {
         /* Check if the section info was protected with a different access mode */
-        if(fspace->sinfo_protected && fspace->sinfo_accmode != H5AC_WRITE)
+        if(fspace->sinfo_protected && (0 != ((fspace->sinfo_accmode) & H5AC__READ_ONLY_FLAG)))
             HGOTO_ERROR(H5E_FSPACE, H5E_CANTDIRTY, FAIL, "attempt to modify read-only section info")
 
         /* If we modified the section info, mark it dirty */
@@ -444,8 +450,10 @@ HDfprintf(stderr, "%s: Relinquishing section info ownership\n", FUNC);
 HDfprintf(stderr, "%s: Freeing section info on disk, old_sect_addr = %a, old_alloc_sect_size = %Hu\n", FUNC, old_sect_addr, old_alloc_sect_size);
 #endif /* H5FS_SINFO_DEBUG */
             /* Release space for section info in file */
-            if(H5MF_xfree(f, H5FD_MEM_FSPACE_SINFO, dxpl_id, old_sect_addr, old_alloc_sect_size) < 0)
-                HGOTO_ERROR(H5E_FSPACE, H5E_CANTFREE, FAIL, "unable to free free space sections")
+            if(!H5F_IS_TMP_ADDR(f, old_sect_addr)) {
+                if(H5MF_xfree(f, H5FD_MEM_FSPACE_SINFO, dxpl_id, old_sect_addr, old_alloc_sect_size) < 0)
+                    HGOTO_ERROR(H5E_FSPACE, H5E_CANTFREE, FAIL, "unable to free free space sections")
+            } /* end if */
         } /* end if */
     } /* end if */
 
@@ -458,14 +466,13 @@ HDfprintf(stderr, "%s: Leaving, ret_value = %d\n", FUNC, ret_value);
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_serialize_size
+ * Function:    H5FS_sect_serialize_size
  *
- * Purpose:	Determine serialized size of all sections in free space manager
+ * Purpose:     Determine serialized size of all sections in free space manager
  *
- * Return:	Success:	non-negative
- *		Failure:	negative
+ * Return:      SUCCEED/FAIL
  *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Monday, May  8, 2006
  *
  *-------------------------------------------------------------------------
@@ -522,15 +529,14 @@ HDfprintf(stderr, "%s: fspace->serial_sect_count = %Hu\n", "H5FS_sect_serialize_
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_increase
+ * Function:    H5FS_sect_increase
  *
- * Purpose:	Increase the size of the serialized free space section info
+ * Purpose:     Increase the size of the serialized free space section info
  *              on disk
  *
- * Return:	Success:	non-negative
- *		Failure:	negative
+ * Return:      SUCCEED/FAIL
  *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Monday, May  8, 2006
  *
  *-------------------------------------------------------------------------
@@ -584,16 +590,14 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_decrease
+ * Function:    H5FS_sect_decrease
  *
- * Purpose:	Decrease the size of the serialized free space section info
+ * Purpose:     Decrease the size of the serialized free space section info
  *              on disk
  *
- * Return:	Success:	non-negative
+ * Return:      SUCCEED/FAIL
  *
- *		Failure:	negative
- *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Monday, May  8, 2006
  *
  *-------------------------------------------------------------------------
@@ -643,14 +647,13 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_size_node_decr
+ * Function:    H5FS_size_node_decr
  *
- * Purpose:	Decrement the number of sections of a particular size
+ * Purpose:     Decrement the number of sections of a particular size
  *
- * Return:	Success:	non-negative
- *		Failure:	negative
+ * Return:      SUCCEED/FAIL
  *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Wednesday, May 17, 2006
  *
  *-------------------------------------------------------------------------
@@ -731,15 +734,14 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_unlink_size
+ * Function:    H5FS_sect_unlink_size
  *
- * Purpose:	Remove a section node from size tracking data structures for
+ * Purpose:     Remove a section node from size tracking data structures for
  *              a free space manager
  *
- * Return:	Success:	non-negative
- *		Failure:	negative
+ * Return:      SUCCEED/FAIL
  *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Wednesday, May 17, 2006
  *
  *-------------------------------------------------------------------------
@@ -786,17 +788,15 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_unlink_rest
+ * Function:    H5FS_sect_unlink_rest
  *
- * Purpose:	Finish unlinking a section from the rest of the free space
+ * Purpose:     Finish unlinking a section from the rest of the free space
  *              manager's data structures, after the section has been removed
  *              from the size tracking data structures
  *
- * Return:	Success:	non-negative
+ * Return:      SUCCEED/FAIL
  *
- *		Failure:	negative
- *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Wednesday, May 17, 2006
  *
  *-------------------------------------------------------------------------
@@ -843,14 +843,13 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_remove_real
+ * Function:    H5FS_sect_remove_real
  *
- * Purpose:	Remove a section from the free space manager
+ * Purpose:     Remove a section from the free space manager
  *
- * Return:	Success:	non-negative
- *		Failure:	negative
+ * Return:      SUCCEED/FAIL
  *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Wednesday, May 17, 2006
  *
  *-------------------------------------------------------------------------
@@ -885,14 +884,13 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_remove
+ * Function:    H5FS_sect_remove
  *
- * Purpose:	Remove a section from the free space manager
+ * Purpose:     Remove a section from the free space manager
  *
- * Return:	Success:	non-negative
- *		Failure:	negative
+ * Return:      SUCCEED/FAIL
  *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Wednesday, May 17, 2006
  *
  *-------------------------------------------------------------------------
@@ -912,7 +910,7 @@ H5FS_sect_remove(H5F_t *f, hid_t dxpl_id, H5FS_t *fspace,
     HDassert(sect);
 
     /* Get a pointer to the section info */
-    if(H5FS_sinfo_lock(f, dxpl_id, fspace, H5AC_WRITE) < 0)
+    if(H5FS_sinfo_lock(f, dxpl_id, fspace, H5AC__NO_FLAGS_SET) < 0)
         HGOTO_ERROR(H5E_FSPACE, H5E_CANTGET, FAIL, "can't get section info")
     sinfo_valid = TRUE;
 
@@ -930,14 +928,13 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_link_size
+ * Function:    H5FS_sect_link_size
  *
- * Purpose:	Add a section of free space to the free list bins
+ * Purpose:     Add a section of free space to the free list bins
  *
- * Return:	Success:	non-negative
- *		Failure:	negative
+ * Return:      SUCCEED/FAIL
  *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Monday, March 20, 2006
  *
  *-------------------------------------------------------------------------
@@ -1038,16 +1035,14 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_link_rest
+ * Function:    H5FS_sect_link_rest
  *
- * Purpose:	Link a section into the rest of the non-size tracking
+ * Purpose:     Link a section into the rest of the non-size tracking
  *              free space manager data structures
  *
- * Return:	Success:	non-negative
+ * Return:      SUCCEED/FAIL
  *
- *		Failure:	negative
- *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Wednesday, May 17, 2006
  *
  *-------------------------------------------------------------------------
@@ -1090,15 +1085,13 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_link
+ * Function:    H5FS_sect_link
  *
- * Purpose:	Link a section into the internal data structures
+ * Purpose:     Link a section into the internal data structures
  *
- * Return:	Success:	non-negative
+ * Return:      SUCCEED/FAIL
  *
- *		Failure:	negative
- *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Wednesday, May 17, 2006
  *
  *-------------------------------------------------------------------------
@@ -1142,24 +1135,16 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_merge
+ * Function:    H5FS_sect_merge
  *
- * Purpose:	Attempt to merge a returned free space section with existing
+ * Purpose:     Attempt to merge a returned free space section with existing
  *              free space.
  *
- * Return:	Success:	non-negative
- *		Failure:	negative
+ * Return:      SUCCEED/FAIL
  *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Wednesday, May 17, 2006
  *
- * Modifications: Vailin Choi; Sept 25th 2008
- *	Changes to the "shrinking" part--
- *	1. Get last section node in merge-list instead of "less-than"
- *	   node for further iteration
- *	2. Remove "can-be-shrunk" section from free-space instead of
- *	   "less-than" section
- *
  *-------------------------------------------------------------------------
  */
 static herr_t
@@ -1167,7 +1152,7 @@ H5FS_sect_merge(H5FS_t *fspace, H5FS_section_info_t **sect, void *op_data)
 {
     H5FS_section_class_t *sect_cls;     /* Section's class */
     hbool_t modified;                   /* Flag to indicate merge or shrink occurred */
-    hbool_t remove_sect = FALSE;       /* Whether a section should be removed before shrinking */
+    hbool_t remove_sect = FALSE;        /* Whether a section should be removed before shrinking */
     htri_t status;                      /* Status value */
     herr_t ret_value = SUCCEED;         /* Return value */
 
@@ -1299,13 +1284,13 @@ HDfprintf(stderr, "%s: Done merging, (*sect) = {%a, %Hu, %u, %s}\n", FUNC, (*sec
 HDfprintf(stderr, "%s: Can shrink!\n", FUNC);
 #endif /* QAK */
 
-		/* Remove SECT from free-space manager */
+                /* Remove SECT from free-space manager */
                 /* (only possible to happen on second+ pass through loop) */
-		if(remove_sect) {
-		    if(H5FS_sect_remove_real(fspace, *sect) < 0)
-			HGOTO_ERROR(H5E_FSPACE, H5E_CANTRELEASE, FAIL, "can't remove section from internal data structures")
-		    remove_sect = FALSE;
-		} /* end if */
+                if(remove_sect) {
+                    if(H5FS_sect_remove_real(fspace, *sect) < 0)
+                        HGOTO_ERROR(H5E_FSPACE, H5E_CANTRELEASE, FAIL, "can't remove section from internal data structures")
+                    remove_sect = FALSE;
+                } /* end if */
 
                 /* Shrink the container */
                 /* (callback can indicate that it has discarded the section by setting *sect to NULL) */
@@ -1329,7 +1314,7 @@ HDfprintf(stderr, "%s: Can shrink!\n", FUNC);
                             remove_sect = TRUE;
                         } /* end if */
                     } /* end if */
-		} /* end if */
+                } /* end if */
 
                 /* Indicate successful merge occurred */
                 modified = TRUE;
@@ -1339,7 +1324,7 @@ HDfprintf(stderr, "%s: Can shrink!\n", FUNC);
 
     /* Check for section that was shrunk away and next section not shrinking */
     if(remove_sect && (*sect != NULL))
-	*sect = NULL;
+        *sect = NULL;
 
 #ifdef QAK
 HDfprintf(stderr, "%s: Done shrinking\n", FUNC);
@@ -1358,15 +1343,13 @@ HDfprintf(stderr, "%s: Leaving, ret_value = %d\n", FUNC, ret_value);
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_add
+ * Function:    H5FS_sect_add
  *
- * Purpose:	Add a section of free space to the free list
+ * Purpose:     Add a section of free space to the free list
  *
- * Return:	Success:	non-negative
+ * Return:      SUCCEED/FAIL
  *
- *		Failure:	negative
- *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Tuesday, March  7, 2006
  *
  *-------------------------------------------------------------------------
@@ -1393,7 +1376,7 @@ HDfprintf(stderr, "%s: *sect = {%a, %Hu, %u, %s}\n", FUNC, sect->addr, sect->siz
     HDassert(sect->size);
 
     /* Get a pointer to the section info */
-    if(H5FS_sinfo_lock(f, dxpl_id, fspace, H5AC_WRITE) < 0)
+    if(H5FS_sinfo_lock(f, dxpl_id, fspace, H5AC__NO_FLAGS_SET) < 0)
         HGOTO_ERROR(H5E_FSPACE, H5E_CANTGET, FAIL, "can't get section info")
     sinfo_valid = TRUE;
 
@@ -1438,7 +1421,7 @@ done:
 
 #ifdef H5FS_DEBUG_ASSERT
 if(!(flags & (H5FS_ADD_DESERIALIZING | H5FS_ADD_SKIP_VALID)))
-    H5FS_assert(fspace);
+    H5FS_assert(fspace, dxpl_id);
 #endif /* H5FS_DEBUG_ASSERT */
 #ifdef H5FS_SINFO_DEBUG
 HDfprintf(stderr, "%s: Leaving, ret_value = %d\n", FUNC, ret_value);
@@ -1448,14 +1431,13 @@ HDfprintf(stderr, "%s: Leaving, ret_value = %d\n", FUNC, ret_value);
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_try_extend
+ * Function:    H5FS_sect_try_extend
  *
- * Purpose:	Try to extend a block using space from a section on the free list
+ * Purpose:     Try to extend a block using space from a section on the free list
  *
- * Return:	Success:	non-negative
- *		Failure:	negative
+ * Return:      SUCCEED/FAIL
  *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Tuesday, January  8, 2008
  *
  *-------------------------------------------------------------------------
@@ -1491,7 +1473,7 @@ HDfprintf(stderr, "%s: fspace->ghost_sect_count = %Hu\n", FUNC, fspace->ghost_se
         H5FS_section_info_t *sect;      /* Temporary free space section */
 
         /* Get a pointer to the section info */
-        if(H5FS_sinfo_lock(f, dxpl_id, fspace, H5AC_WRITE) < 0)
+        if(H5FS_sinfo_lock(f, dxpl_id, fspace, H5AC__NO_FLAGS_SET) < 0)
             HGOTO_ERROR(H5E_FSPACE, H5E_CANTGET, FAIL, "can't get section info")
         sinfo_valid = TRUE;
 
@@ -1579,23 +1561,82 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_find_node
+ * Function:    H5FS_sect_try_merge
  *
- * Purpose:	Locate a section of free space (in existing free space list
+ * Purpose:     Try to merge/shrink a block
+ *
+ * Return:      TRUE:       merged/shrunk
+ *              FALSE:      not merged/not shrunk
+ *              Failure:    negative
+ *
+ * Programmer:  Vailin Choi
+ *              June 10, 2009
+ *
+ *-------------------------------------------------------------------------
+ */
+htri_t
+H5FS_sect_try_merge(H5F_t *f, hid_t dxpl_id, H5FS_t *fspace, H5FS_section_info_t *sect,
+    unsigned flags, void *op_data)
+{
+    hbool_t sinfo_valid = FALSE;        /* Whether the section info is valid */
+    hbool_t sinfo_modified = FALSE;     /* Whether the section info was modified */
+    hsize_t saved_fs_size;              /* Copy of the free-space section size */
+    htri_t ret_value = FALSE;           /* Return value */
+
+    FUNC_ENTER_NOAPI(FAIL)
+
+    /* Check arguments. */
+    HDassert(f);
+    HDassert(fspace);
+    HDassert(sect);
+    HDassert(H5F_addr_defined(sect->addr));
+    HDassert(sect->size);
+
+    /* Get a pointer to the section info */
+    if(H5FS_sinfo_lock(f, dxpl_id, fspace, H5AC__NO_FLAGS_SET) < 0)
+        HGOTO_ERROR(H5E_FSPACE, H5E_CANTGET, FAIL, "can't get section info")
+    sinfo_valid = TRUE;
+    saved_fs_size = sect->size;
+
+    /* Attempt to merge/shrink section with existing sections */
+    if(H5FS_sect_merge(fspace, &sect, op_data) < 0)
+        HGOTO_ERROR(H5E_FSPACE, H5E_CANTMERGE, FAIL, "can't merge sections")
+
+    /* Check if section is shrunk and/or merged away completely */
+    if(!sect) {
+        sinfo_modified = TRUE;
+        HGOTO_DONE(TRUE)
+    } /* end if */
+    else {
+        /* Check if section is merged */
+        if(sect->size > saved_fs_size) {
+            if(H5FS_sect_link(fspace, sect, flags) < 0)
+                HGOTO_ERROR(H5E_FSPACE, H5E_CANTINSERT, FAIL, "can't insert free space section into skip list")
+            sinfo_modified = TRUE;
+            HGOTO_DONE(TRUE)
+        } /* end if */
+    } /* end else */
+
+done:
+    /* Release the section info */
+    if(sinfo_valid && H5FS_sinfo_unlock(f, dxpl_id, fspace, sinfo_modified) < 0)
+        HDONE_ERROR(H5E_FSPACE, H5E_CANTRELEASE, FAIL, "can't release section info")
+
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* H5FS_sect_try_merge() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:    H5FS_sect_find_node
+ *
+ * Purpose:     Locate a section of free space (in existing free space list
  *              bins) that is large enough to fulfill request.
  *
- * Return:	Success:	non-negative
+ * Return:      SUCCEED/FAIL
  *
- *		Failure:	negative
- *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Monday, March 20, 2006
  *
- * Modifications:
- *	Vailin Choi, July 29th, 2008
- *	  Modified to handle alignment by going through each bin to find
- *	  a section that is big enough to fulfill "request+fragment for alignment"
- *
  *-------------------------------------------------------------------------
  */
 static htri_t
@@ -1627,34 +1668,34 @@ HDfprintf(stderr, "%s: bin = %u\n", FUNC, bin);
 #endif /* QAK */
     alignment = fspace->alignment;
     if(!((alignment > 1) && (request >= fspace->threshold)))
-	alignment = 0; /* no alignment */
+        alignment = 0; /* no alignment */
 
     do {
         /* Check if there's any sections in this bin */
-	if(fspace->sinfo->bins[bin].bin_list) {
+        if(fspace->sinfo->bins[bin].bin_list) {
 
-	    if (!alignment) { /* no alignment */
-		/* Find the first free space section that is large enough to fulfill request */
-		/* (Since the bins use skip lists to track the sizes of the address-ordered
-		 *  lists, this is actually a "best fit" algorithm)
-		 */
-		/* Look for large enough free space section in this bin */
-		if((fspace_node = (H5FS_node_t *)H5SL_greater(fspace->sinfo->bins[bin].bin_list, &request))) {
-		    /* Take first node off of the list (ie. node w/lowest address) */
-		    if(NULL == (*node = (H5FS_section_info_t *)H5SL_remove_first(fspace_node->sect_list)))
-			HGOTO_ERROR(H5E_FSPACE, H5E_CANTREMOVE, FAIL, "can't remove free space node from skip list")
+            if (!alignment) { /* no alignment */
+                /* Find the first free space section that is large enough to fulfill request */
+                /* (Since the bins use skip lists to track the sizes of the address-ordered
+                 *  lists, this is actually a "best fit" algorithm)
+                 */
+                /* Look for large enough free space section in this bin */
+                if((fspace_node = (H5FS_node_t *)H5SL_greater(fspace->sinfo->bins[bin].bin_list, &request))) {
+                    /* Take first node off of the list (ie. node w/lowest address) */
+                    if(NULL == (*node = (H5FS_section_info_t *)H5SL_remove_first(fspace_node->sect_list)))
+                        HGOTO_ERROR(H5E_FSPACE, H5E_CANTREMOVE, FAIL, "can't remove free space node from skip list")
 
-		    /* Get section's class */
-		    cls = &fspace->sect_cls[(*node)->type];
-		    /* Decrement # of sections in section size node */
-		    if(H5FS_size_node_decr(fspace->sinfo, bin, fspace_node, cls) < 0)
-			HGOTO_ERROR(H5E_FSPACE, H5E_CANTREMOVE, FAIL, "can't remove free space size node from skip list")
-		    if(H5FS_sect_unlink_rest(fspace, cls, *node) < 0)
-			HGOTO_ERROR(H5E_FSPACE, H5E_CANTFREE, FAIL, "can't remove section from non-size tracking data structures")
-		    /* Indicate that we found a node for the request */
-		    HGOTO_DONE(TRUE)
-		} /* end if */
-	    }  /* end if */
+                    /* Get section's class */
+                    cls = &fspace->sect_cls[(*node)->type];
+                    /* Decrement # of sections in section size node */
+                    if(H5FS_size_node_decr(fspace->sinfo, bin, fspace_node, cls) < 0)
+                        HGOTO_ERROR(H5E_FSPACE, H5E_CANTREMOVE, FAIL, "can't remove free space size node from skip list")
+                    if(H5FS_sect_unlink_rest(fspace, cls, *node) < 0)
+                        HGOTO_ERROR(H5E_FSPACE, H5E_CANTFREE, FAIL, "can't remove section from non-size tracking data structures")
+                    /* Indicate that we found a node for the request */
+                    HGOTO_DONE(TRUE)
+                } /* end if */
+            }  /* end if */
             else { /* alignment is set */
                 /* get the first node of a certain size in this bin */
                 curr_size_node = H5SL_first(fspace->sinfo->bins[bin].bin_list);
@@ -1723,9 +1764,9 @@ HDfprintf(stderr, "%s: bin = %u\n", FUNC, bin);
                     /* Get the next size node in the bin */
                     curr_size_node = H5SL_next(curr_size_node);
                 } /* end while of curr_size_node */
-	    }  /* else of alignment */
-	} /* if bin_list */
-	/* Advance to next larger bin */
+            }  /* else of alignment */
+        } /* if bin_list */
+        /* Advance to next larger bin */
         bin++;
     } while(bin < fspace->sinfo->nbins);
 
@@ -1735,22 +1776,16 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_find
+ * Function:    H5FS_sect_find
  *
- * Purpose:	Locate a section of free space (in existing free space list) that
+ * Purpose:     Locate a section of free space (in existing free space list) that
  *              is large enough to fulfill request.
  *
- * Return:	Success:	non-negative
+ * Return:      SUCCEED/FAIL
  *
- *		Failure:	negative
- *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Tuesday, March  7, 2006
  *
- * Modifications:
- *	Vailin Choi, July 29th 2008
- *	  Move H5FS_sect_unlink_rest() to H5FS_sect_find_node()
- *
  *-------------------------------------------------------------------------
  */
 htri_t
@@ -1781,7 +1816,7 @@ HDfprintf(stderr, "%s: fspace->ghost_sect_count = %Hu\n", FUNC, fspace->ghost_se
 #endif /* QAK */
     if(fspace->tot_sect_count > 0) {
         /* Get a pointer to the section info */
-        if(H5FS_sinfo_lock(f, dxpl_id, fspace, H5AC_WRITE) < 0)
+        if(H5FS_sinfo_lock(f, dxpl_id, fspace, H5AC__NO_FLAGS_SET) < 0)
             HGOTO_ERROR(H5E_FSPACE, H5E_CANTGET, FAIL, "can't get section info")
         sinfo_valid = TRUE;
 
@@ -1796,7 +1831,7 @@ HDfprintf(stderr, "%s: fspace->ghost_sect_count = %Hu\n", FUNC, fspace->ghost_se
 #ifdef QAK
 HDfprintf(stderr, "%s: (*node)->size = %Hu, (*node)->addr = %a, (*node)->type = %u\n", FUNC, (*node)->size, (*node)->addr, (*node)->type);
 #endif /* QAK */
-	}
+        }
     } /* end if */
 
 done:
@@ -1805,29 +1840,27 @@ done:
         HDONE_ERROR(H5E_FSPACE, H5E_CANTRELEASE, FAIL, "can't release section info")
 
 #ifdef H5FS_DEBUG_ASSERT
-    H5FS_assert(fspace);
+    H5FS_assert(fspace, dxpl_id);
 #endif /* H5FS_DEBUG_ASSERT */
     FUNC_LEAVE_NOAPI(ret_value)
 } /* H5FS_sect_find() */
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_iterate_sect_cb
+ * Function:    H5FS_iterate_sect_cb
  *
- * Purpose:	Skip list iterator callback to iterate over free space sections
+ * Purpose:     Skip list iterator callback to iterate over free space sections
  *              of a particular size
  *
- * Return:	Success:	non-negative
+ * Return:      SUCCEED/FAIL
  *
- *		Failure:	negative
- *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Saturday, May 13, 2006
  *
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5FS_iterate_sect_cb(void *_item, void UNUSED *key, void *_udata)
+H5FS_iterate_sect_cb(void *_item, void H5_ATTR_UNUSED *key, void *_udata)
 {
     H5FS_section_info_t *sect_info = (H5FS_section_info_t *)_item;   /* Free space section to work on */
     H5FS_iter_ud_t *udata = (H5FS_iter_ud_t *)_udata; /* Callback info */
@@ -1850,22 +1883,20 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_iterate_node_cb
+ * Function:    H5FS_iterate_node_cb
  *
- * Purpose:	Skip list iterator callback to iterate over free space sections
+ * Purpose:     Skip list iterator callback to iterate over free space sections
  *              in a bin
  *
- * Return:	Success:	non-negative
+ * Return:      SUCCEED/FAIL
  *
- *		Failure:	negative
- *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Saturday, May 13, 2006
  *
  *-------------------------------------------------------------------------
  */
 static herr_t
-H5FS_iterate_node_cb(void *_item, void UNUSED *key, void *_udata)
+H5FS_iterate_node_cb(void *_item, void H5_ATTR_UNUSED *key, void *_udata)
 {
     H5FS_node_t *fspace_node = (H5FS_node_t *)_item;   /* Free space size node to work on */
     H5FS_iter_ud_t *udata = (H5FS_iter_ud_t *)_udata; /* Callback info */
@@ -1889,15 +1920,13 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_iterate
+ * Function:    H5FS_sect_iterate
  *
- * Purpose:	Iterate over all the sections managed
+ * Purpose:     Iterate over all the sections managed
  *
- * Return:	Success:	non-negative
+ * Return:      SUCCEED/FAIL
  *
- *		Failure:	negative
- *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Saturday, May 13, 2006
  *
  *-------------------------------------------------------------------------
@@ -1929,7 +1958,7 @@ HDfprintf(stderr, "%s: fspace->tot_sect_count = %Hu\n", FUNC, fspace->tot_sect_c
         unsigned bin;           /* Current bin we are on */
 
         /* Get a pointer to the section info */
-        if(H5FS_sinfo_lock(f, dxpl_id, fspace, H5AC_READ) < 0)
+        if(H5FS_sinfo_lock(f, dxpl_id, fspace, H5AC__READ_ONLY_FLAG) < 0)
             HGOTO_ERROR(H5E_FSPACE, H5E_CANTGET, FAIL, "can't get section info")
         sinfo_valid = TRUE;
 
@@ -1957,14 +1986,13 @@ done:
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_stats
+ * Function:    H5FS_sect_stats
  *
- * Purpose:	Retrieve info about the sections managed
+ * Purpose:     Retrieve info about the sections managed
  *
- * Return:	Success:	non-negative
- *		Failure:	negative
+ * Return:      SUCCEED/FAIL
  *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Tuesday, May 30, 2006
  *
  *-------------------------------------------------------------------------
@@ -1988,23 +2016,21 @@ H5FS_sect_stats(const H5FS_t *fspace, hsize_t *tot_space, hsize_t *nsects)
 
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_change_class
+ * Function:    H5FS_sect_change_class
  *
- * Purpose:	Make appropriate adjustments to internal data structures when
+ * Purpose:     Make appropriate adjustments to internal data structures when
  *              a section changes class
  *
- * Return:	Success:	non-negative
+ * Return:      SUCCEED/FAIL
  *
- *		Failure:	negative
- *
- * Programmer:	Quincey Koziol
+ * Programmer:  Quincey Koziol
  *              Monday, July 10, 2006
  *
  *-------------------------------------------------------------------------
  */
 herr_t
 H5FS_sect_change_class(H5F_t *f, hid_t dxpl_id, H5FS_t *fspace,
-    H5FS_section_info_t *sect, unsigned new_class)
+    H5FS_section_info_t *sect, uint16_t new_class)
 {
     const H5FS_section_class_t *old_cls;        /* Old class of section */
     const H5FS_section_class_t *new_cls;        /* New class of section */
@@ -2021,7 +2047,7 @@ H5FS_sect_change_class(H5F_t *f, hid_t dxpl_id, H5FS_t *fspace,
     HDassert(new_class < fspace->nclasses);
 
     /* Get a pointer to the section info */
-    if(H5FS_sinfo_lock(f, dxpl_id, fspace, H5AC_WRITE) < 0)
+    if(H5FS_sinfo_lock(f, dxpl_id, fspace, H5AC__NO_FLAGS_SET) < 0)
         HGOTO_ERROR(H5E_FSPACE, H5E_CANTGET, FAIL, "can't get section info")
     sinfo_valid = TRUE;
 
@@ -2160,20 +2186,19 @@ done:
 #ifdef H5FS_DEBUG_ASSERT
 
 /*-------------------------------------------------------------------------
- * Function:	H5FS_sect_assert
+ * Function:    H5FS_sect_assert
  *
- * Purpose:	Verify that the sections managed are mostly sane
+ * Purpose:     Verify that the sections managed are mostly sane
  *
- * Return:	Non-negative on success, negative on failure
+ * Return:      SUCCEED/FAIL
  *
- * Programmer:	Quincey Koziol
- *		koziol@hdfgroup.org
- *		Jul 17 2006
+ * Programmer:  Quincey Koziol
+ *              Jul 17 2006
  *
  *-------------------------------------------------------------------------
  */
 herr_t
-H5FS_sect_assert(const H5FS_t *fspace)
+H5FS_sect_assert(const H5FS_t *fspace, hid_t dxpl_id)
 {
     hsize_t separate_obj;       /* The number of separate objects managed */
 
@@ -2245,7 +2270,7 @@ HDfprintf(stderr, "%s: sect->size = %Hu, sect->addr = %a, sect->type = %u\n", "H
                         HDassert(H5F_addr_defined(sect->addr));
                         HDassert(fspace_node->sect_size == sect->size);
                         if(cls->valid)
-                            (*cls->valid)(cls, sect);
+                            (*cls->valid)(cls, sect, dxpl_id);
 
                         /* Add to correct count */
                         if(cls->flags & H5FS_CLS_GHOST_OBJ)
@@ -2335,7 +2360,7 @@ H5FS_sect_try_shrink_eoa(const H5F_t *f, hid_t dxpl_id, const H5FS_t *fspace, vo
     /* Check arguments. */
     HDassert(fspace);
 
-    if(H5FS_sinfo_lock(f, dxpl_id, fspace, H5AC_WRITE) < 0)
+    if(H5FS_sinfo_lock(f, dxpl_id, fspace, H5AC__NO_FLAGS_SET) < 0)
         HGOTO_ERROR(H5E_FSPACE, H5E_CANTGET, FAIL, "can't get section info")
     sinfo_valid = TRUE;
 
