@@ -27,8 +27,9 @@
 /****************/
 /* Module Setup */
 /****************/
-#define H5O_PACKAGE		/*suppress error about including H5Opkg	  */
-#define H5P_PACKAGE		/*suppress error about including H5Ppkg	  */
+
+#define H5O_FRIEND		/*suppress error about including H5Opkg	  */
+#include "H5Pmodule.h"          /* This source code file is part of the H5P module */
 
 
 /***********/
@@ -37,6 +38,7 @@
 #include "H5private.h"		/* Generic Functions			*/
 #include "H5Eprivate.h"		/* Error handling		  	*/
 #include "H5Iprivate.h"		/* IDs			  		*/
+#include "H5MMprivate.h"	/* Memory management			*/
 #include "H5Opkg.h"             /* Object headers			*/
 #include "H5Ppkg.h"		/* Property lists		  	*/
 #include "H5PLprivate.h"	/* Dynamic plugin			*/
@@ -50,13 +52,26 @@
 /* ========= Object Creation properties ============ */
 /* Definitions for the max. # of attributes to store compactly */
 #define H5O_CRT_ATTR_MAX_COMPACT_SIZE   sizeof(unsigned)
+#define H5O_CRT_ATTR_MAX_COMPACT_ENC    H5P__encode_unsigned
+#define H5O_CRT_ATTR_MAX_COMPACT_DEC    H5P__decode_unsigned
 /* Definitions for the min. # of attributes to store densely */
 #define H5O_CRT_ATTR_MIN_DENSE_SIZE     sizeof(unsigned)
+#define H5O_CRT_ATTR_MIN_DENSE_ENC      H5P__encode_unsigned
+#define H5O_CRT_ATTR_MIN_DENSE_DEC      H5P__decode_unsigned
 /* Definitions for object header flags */
 #define H5O_CRT_OHDR_FLAGS_SIZE         sizeof(uint8_t)
+#define H5O_CRT_OHDR_FLAGS_ENC          H5P__encode_uint8_t
+#define H5O_CRT_OHDR_FLAGS_DEC          H5P__decode_uint8_t
 /* Definitions for filter pipeline */
 #define H5O_CRT_PIPELINE_SIZE sizeof(H5O_pline_t)
+#define H5O_CRT_PIPELINE_SET            H5P__ocrt_pipeline_set
+#define H5O_CRT_PIPELINE_GET            H5P__ocrt_pipeline_get
+#define H5O_CRT_PIPELINE_ENC            H5P__ocrt_pipeline_enc
+#define H5O_CRT_PIPELINE_DEC            H5P__ocrt_pipeline_dec
+#define H5O_CRT_PIPELINE_DEL            H5P__ocrt_pipeline_del
+#define H5O_CRT_PIPELINE_COPY           H5P__ocrt_pipeline_copy
 #define H5O_CRT_PIPELINE_CMP            H5P__ocrt_pipeline_cmp
+#define H5O_CRT_PIPELINE_CLOSE          H5P__ocrt_pipeline_close
 
 
 /******************/
@@ -75,11 +90,16 @@
 
 /* Property class callbacks */
 static herr_t H5P__ocrt_reg_prop(H5P_genclass_t *pclass);
-static herr_t H5P__ocrt_copy(hid_t new_plist_t, hid_t old_plist_t, void *copy_data);
-static herr_t H5P__ocrt_close(hid_t dxpl_id, void *close_data);
 
 /* Property callbacks */
+static herr_t H5P__ocrt_pipeline_enc(const void *value, void **_pp, size_t *size);
+static herr_t H5P__ocrt_pipeline_dec(const void **_pp, void *value);
+static herr_t H5P__ocrt_pipeline_set(hid_t prop_id, const char *name, size_t size, void *value);
+static herr_t H5P__ocrt_pipeline_get(hid_t prop_id, const char *name, size_t size, void *value);
+static herr_t H5P__ocrt_pipeline_del(hid_t prop_id, const char *name, size_t size, void *value);
+static herr_t H5P__ocrt_pipeline_copy(const char *name, size_t size, void *value);
 static int H5P__ocrt_pipeline_cmp(const void *value1, const void *value2, size_t size);
+static herr_t H5P__ocrt_pipeline_close(const char *name, size_t size, void *value);
 
 /* Local routines */
 static herr_t H5P__set_filter(H5P_genplist_t *plist, H5Z_filter_t filter, 
@@ -102,9 +122,9 @@ const H5P_libclass_t H5P_CLS_OCRT[1] = {{
 
     NULL,		        /* Class creation callback      */
     NULL,		        /* Class creation callback info */
-    H5P__ocrt_copy,		/* Class copy callback          */
+    NULL,			/* Class copy callback          */
     NULL,		        /* Class copy callback info     */
-    H5P__ocrt_close,		/* Class close callback         */
+    NULL,			/* Class close callback         */
     NULL 		        /* Class close callback info    */
 }};
 
@@ -119,6 +139,12 @@ const H5P_libclass_t H5P_CLS_OCRT[1] = {{
 /* Local Variables */
 /*******************/
 
+/* Property value defaults */
+static const unsigned H5O_def_attr_max_compact_g = H5O_CRT_ATTR_MAX_COMPACT_DEF;   /* Default max. compact attribute storage settings */
+static const unsigned H5O_def_attr_min_dense_g = H5O_CRT_ATTR_MIN_DENSE_DEF;       /* Default min. dense attribute storage settings */
+static const uint8_t H5O_def_ohdr_flags_g = H5O_CRT_OHDR_FLAGS_DEF;        /* Default object header flag settings */
+static const H5O_pline_t H5O_def_pline_g = H5O_CRT_PIPELINE_DEF;           /* Default I/O pipeline setting */
+
 
 
 /*-------------------------------------------------------------------------
@@ -136,125 +162,38 @@ const H5P_libclass_t H5P_CLS_OCRT[1] = {{
 static herr_t
 H5P__ocrt_reg_prop(H5P_genclass_t *pclass)
 {
-    unsigned attr_max_compact = H5O_CRT_ATTR_MAX_COMPACT_DEF;   /* Default max. compact attribute storage settings */
-    unsigned attr_min_dense = H5O_CRT_ATTR_MIN_DENSE_DEF;       /* Default min. dense attribute storage settings */
-    uint8_t ohdr_flags = H5O_CRT_OHDR_FLAGS_DEF;        /* Default object header flag settings */
-    H5O_pline_t pline = H5O_CRT_PIPELINE_DEF;           /* Default I/O pipeline setting */
     herr_t ret_value = SUCCEED;         /* Return value */
 
     FUNC_ENTER_STATIC
 
     /* Register max. compact attribute storage property */
-    if(H5P_register_real(pclass, H5O_CRT_ATTR_MAX_COMPACT_NAME, H5O_CRT_ATTR_MAX_COMPACT_SIZE, &attr_max_compact, NULL, NULL, NULL, NULL, NULL, NULL, NULL) < 0)
+    if(H5P_register_real(pclass, H5O_CRT_ATTR_MAX_COMPACT_NAME, H5O_CRT_ATTR_MAX_COMPACT_SIZE, &H5O_def_attr_max_compact_g, 
+            NULL, NULL, NULL, H5O_CRT_ATTR_MAX_COMPACT_ENC, H5O_CRT_ATTR_MAX_COMPACT_DEC,
+            NULL, NULL, NULL, NULL) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTINSERT, FAIL, "can't insert property into class")
 
     /* Register min. dense attribute storage property */
-    if(H5P_register_real(pclass, H5O_CRT_ATTR_MIN_DENSE_NAME, H5O_CRT_ATTR_MIN_DENSE_SIZE, &attr_min_dense, NULL, NULL, NULL, NULL, NULL, NULL, NULL) < 0)
+    if(H5P_register_real(pclass, H5O_CRT_ATTR_MIN_DENSE_NAME, H5O_CRT_ATTR_MIN_DENSE_SIZE, &H5O_def_attr_min_dense_g, 
+            NULL, NULL, NULL, H5O_CRT_ATTR_MIN_DENSE_ENC, H5O_CRT_ATTR_MIN_DENSE_DEC,
+            NULL, NULL, NULL, NULL) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTINSERT, FAIL, "can't insert property into class")
 
     /* Register object header flags property */
-    if(H5P_register_real(pclass, H5O_CRT_OHDR_FLAGS_NAME, H5O_CRT_OHDR_FLAGS_SIZE, &ohdr_flags, NULL, NULL, NULL, NULL, NULL, NULL, NULL) < 0)
+    if(H5P_register_real(pclass, H5O_CRT_OHDR_FLAGS_NAME, H5O_CRT_OHDR_FLAGS_SIZE, &H5O_def_ohdr_flags_g, 
+            NULL, NULL, NULL, H5O_CRT_OHDR_FLAGS_ENC, H5O_CRT_OHDR_FLAGS_DEC,
+            NULL, NULL, NULL, NULL) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTINSERT, FAIL, "can't insert property into class")
 
     /* Register the pipeline property */
-    if(H5P_register_real(pclass, H5O_CRT_PIPELINE_NAME, H5O_CRT_PIPELINE_SIZE, &pline, NULL, NULL, NULL, NULL, NULL, H5O_CRT_PIPELINE_CMP, NULL) < 0)
+    if(H5P_register_real(pclass, H5O_CRT_PIPELINE_NAME, H5O_CRT_PIPELINE_SIZE, &H5O_def_pline_g, 
+            NULL, H5O_CRT_PIPELINE_SET, H5O_CRT_PIPELINE_GET, H5O_CRT_PIPELINE_ENC, H5O_CRT_PIPELINE_DEC,
+            H5O_CRT_PIPELINE_DEL, H5O_CRT_PIPELINE_COPY, H5O_CRT_PIPELINE_CMP, H5O_CRT_PIPELINE_CLOSE) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTINSERT, FAIL, "can't insert property into class")
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5P__ocrt_reg_prop() */
 
-
-/*-------------------------------------------------------------------------
- * Function:       H5P__ocrt_copy
- *
- * Purpose:        Callback routine which is called whenever any object
- *                 creation property list is copied.  This routine copies
- *                 the properties from the old list to the new list.
- *
- * Return:         Success:        Non-negative
- *                 Failure:        Negative
- *
- * Programmer:     Neil Fortner
- *                 Monday, September 21, 2009
- *
- *-------------------------------------------------------------------------
- */
-/* ARGSUSED */
-static herr_t
-H5P__ocrt_copy(hid_t dst_plist_id, hid_t src_plist_id, void UNUSED *copy_data)
-{
-    H5O_pline_t    src_pline, dst_pline;        /* Source & destination pipelines */
-    H5P_genplist_t *src_plist;                  /* Pointer to source property list */
-    H5P_genplist_t *dst_plist;                  /* Pointer to destination property list */
-    herr_t         ret_value = SUCCEED;         /* Return value */
-
-    FUNC_ENTER_STATIC
-
-    /* Verify property list IDs */
-    if(NULL == (dst_plist = (H5P_genplist_t *)H5I_object(dst_plist_id)))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not an object creation property list")
-    if(NULL == (src_plist = (H5P_genplist_t *)H5I_object(src_plist_id)))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not an object creation property list")
-
-    /* Get the link pipeline property from the old property list */
-    if(H5P_get(src_plist, H5O_CRT_PIPELINE_NAME, &src_pline) < 0)
-        HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't get pipeline")
-
-    /* Make copy of link pipeline */
-    if(NULL == H5O_msg_copy(H5O_PLINE_ID, &src_pline, &dst_pline))
-        HGOTO_ERROR(H5E_PLIST, H5E_CANTINIT, FAIL, "can't copy link pipeline")
-
-    /* Set the link pipeline property for the destination property list */
-    if(H5P_set(dst_plist, H5O_CRT_PIPELINE_NAME, &dst_pline) < 0)
-        HGOTO_ERROR(H5E_PLIST, H5E_CANTSET, FAIL, "can't set pipeline")
-
-done:
-    FUNC_LEAVE_NOAPI(ret_value)
-} /* end H5P__ocrt_copy() */
-
-
-/*-------------------------------------------------------------------------
- * Function:	H5P__ocrt_close
- *
- * Purpose:	Callback routine which is called whenever any object create
- *              property list is closed.  This routine performs any generic
- *              cleanup needed on the properties the library put into the list.
- *
- * Return:	Success:	Non-negative
- *		Failure:	Negative
- *
- * Programmer:     Neil Fortner
- *                 Monday, September 21, 2009
- *
- *-------------------------------------------------------------------------
- */
-/* ARGSUSED */
-static herr_t
-H5P__ocrt_close(hid_t dcpl_id, void UNUSED *close_data)
-{
-    H5O_pline_t     pline;              /* I/O pipeline */
-    H5P_genplist_t *plist;              /* Property list */
-    herr_t ret_value = SUCCEED;         /* Return value */
-
-    FUNC_ENTER_STATIC
-
-    /* Check arguments */
-    if(NULL == (plist = (H5P_genplist_t *)H5I_object(dcpl_id)))
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "not an object creation property list")
-
-    /* Get the link pipeline property from the old property list */
-    if(H5P_get(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
-        HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't get pipeline")
-
-    /* Clean up any values set for the link pipeline */
-    if(H5O_msg_reset(H5O_PLINE_ID, &pline) < 0)
-        HGOTO_ERROR(H5E_PLIST, H5E_CANTFREE, FAIL, "can't release pipeline info")
-
-done:
-    FUNC_LEAVE_NOAPI(ret_value)
-} /* end H5P__ocrt_close() */
-
 
 /*-------------------------------------------------------------------------
  * Function:	H5Pset_attr_phase_change
@@ -594,7 +533,7 @@ H5P_modify_filter(H5P_genplist_t *plist, H5Z_filter_t filter, unsigned flags,
     FUNC_ENTER_NOAPI(FAIL)
 
     /* Get the pipeline property to modify */
-    if(H5P_get(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+    if(H5P_peek(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't get pipeline")
 
     /* Modify the filter parameters of the I/O pipeline */
@@ -602,7 +541,7 @@ H5P_modify_filter(H5P_genplist_t *plist, H5Z_filter_t filter, unsigned flags,
         HGOTO_ERROR(H5E_PLINE, H5E_CANTINIT, FAIL, "unable to add filter to pipeline")
 
     /* Put the I/O pipeline information back into the property list */
-    if(H5P_set(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+    if(H5P_poke(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTSET, FAIL, "can't set pipeline")
 
 done:
@@ -807,24 +746,20 @@ H5P__set_filter(H5P_genplist_t *plist, H5Z_filter_t filter, unsigned int flags,
 
     /* Check if filter is already available */
     if((filter_avail = H5Z_filter_avail(filter)) < 0)
-        HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't check filter availability")
+        HGOTO_ERROR(H5E_PLIST, H5E_CANTSET, FAIL, "can't check filter availability")
 
     /* If filter is not available, try to dynamically load it */
     if(!filter_avail) {
-#ifndef H5_VMS
         const H5Z_class2_t *filter_info;
 
         if(NULL == (filter_info = (const H5Z_class2_t *)H5PL_load(H5PL_TYPE_FILTER, (int)filter)))
             HGOTO_ERROR(H5E_PLINE, H5E_CANTLOAD, FAIL, "failed to load dynamically loaded plugin")
         if(H5Z_register(filter_info) < 0)
-	    HGOTO_ERROR(H5E_PLINE, H5E_CANTINIT, FAIL, "unable to register dynamic filter")
-#else /*H5_VMS*/
-        HGOTO_ERROR(H5E_PLIST, H5E_CANTSET, FAIL, "filter is NOT registered")
-#endif /*H5_VMS*/
+	    HGOTO_ERROR(H5E_PLINE, H5E_CANTINIT, FAIL, "unable to register filter")
     } /* end if */
 
     /* Get the pipeline property to append to */
-    if(H5P_get(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+    if(H5P_peek(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't get pipeline")
 
     /* Add the filter to the I/O pipeline */
@@ -832,7 +767,7 @@ H5P__set_filter(H5P_genplist_t *plist, H5Z_filter_t filter, unsigned int flags,
         HGOTO_ERROR(H5E_PLINE, H5E_CANTINIT, FAIL, "unable to add filter to pipeline")
 
     /* Put the I/O pipeline information back into the property list */
-    if(H5P_set(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+    if(H5P_poke(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTSET, FAIL, "can't set pipeline")
 
 done:
@@ -882,7 +817,7 @@ H5Pget_nfilters(hid_t plist_id)
         HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "can't find object for ID")
 
     /* Get the pipeline property to query */
-    if(H5P_get(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+    if(H5P_peek(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't get pipeline")
 
     /* Set return value */
@@ -965,7 +900,7 @@ H5Pget_filter2(hid_t plist_id, unsigned idx, unsigned int *flags/*out*/,
         HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, H5Z_FILTER_ERROR, "can't find object for ID")
 
     /* Get the pipeline property to query */
-    if(H5P_get(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+    if(H5P_peek(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, H5Z_FILTER_ERROR, "can't get pipeline")
 
     /* Check index */
@@ -1021,7 +956,7 @@ H5P_get_filter_by_id(H5P_genplist_t *plist, H5Z_filter_t id, unsigned int *flags
     FUNC_ENTER_NOAPI(FAIL)
 
     /* Get pipeline info */
-    if(H5P_get(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+    if(H5P_peek(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't get pipeline")
 
     /* Get pointer to filter in pipeline */
@@ -1072,8 +1007,6 @@ H5Pget_filter_by_id2(hid_t plist_id, H5Z_filter_t id, unsigned int *flags/*out*/
     size_t namelen, char name[]/*out*/, unsigned *filter_config)
 {
     H5P_genplist_t  *plist;                 /* Property list */
-    H5O_pline_t     pline;                  /* Filter pipeline */
-    const H5Z_filter_info_t *filter;        /* Pointer to filter information */
     herr_t          ret_value = SUCCEED;    /* Return value */
 
     FUNC_ENTER_API(FAIL)
@@ -1152,7 +1085,7 @@ H5Pall_filters_avail(hid_t plist_id)
         HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "can't find object for ID")
 
     /* Get the pipeline property to query */
-    if(H5P_get(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+    if(H5P_peek(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't get pipeline")
 
     /* Check if all filters are available */
@@ -1175,7 +1108,7 @@ done:
  *              FAIL: 		error
  *
  * Programmer:	Raymond Lu
- *              14 May 2013
+ *              26 April 2013
  *
  *-------------------------------------------------------------------------
  */
@@ -1188,7 +1121,7 @@ H5P_filter_in_pline(H5P_genplist_t *plist, H5Z_filter_t id)
     FUNC_ENTER_NOAPI(FAIL)
 
     /* Get pipeline info */
-    if(H5P_get(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+    if(H5P_peek(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't get pipeline")
 
     /* Check if the file is in the pipeline */
@@ -1235,7 +1168,7 @@ H5Premove_filter(hid_t plist_id, H5Z_filter_t filter)
         HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "can't find object for ID")
 
     /* Get the pipeline property to modify */
-    if(H5P_get(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+    if(H5P_peek(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't get pipeline")
 
     /* Check if there are any filters */
@@ -1245,7 +1178,7 @@ H5Premove_filter(hid_t plist_id, H5Z_filter_t filter)
             HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't delete filter")
 
         /* Put the I/O pipeline information back into the property list */
-        if(H5P_set(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+        if(H5P_poke(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
             HGOTO_ERROR(H5E_PLIST, H5E_CANTSET, FAIL, "can't set pipeline")
     } /* end if */
 
@@ -1303,7 +1236,7 @@ H5Pset_deflate(hid_t plist_id, unsigned level)
         HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "can't find object for ID")
 
     /* Get the pipeline property to append to */
-    if(H5P_get(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+    if(H5P_peek(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't get pipeline")
 
     /* Add the filter */
@@ -1311,7 +1244,7 @@ H5Pset_deflate(hid_t plist_id, unsigned level)
         HGOTO_ERROR(H5E_PLINE, H5E_CANTINIT, FAIL, "unable to add deflate filter to pipeline")
 
     /* Put the I/O pipeline information back into the property list */
-    if(H5P_set(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+    if(H5P_poke(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTSET, FAIL, "can't set pipeline")
 
 done:
@@ -1354,7 +1287,7 @@ H5Pset_fletcher32(hid_t plist_id)
         HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, FAIL, "can't find object for ID")
 
     /* Get the pipeline property to append to */
-    if(H5P_get(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+    if(H5P_peek(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, FAIL, "can't get pipeline")
 
     /* Add the Fletcher32 checksum as a filter */
@@ -1362,7 +1295,7 @@ H5Pset_fletcher32(hid_t plist_id)
         HGOTO_ERROR(H5E_PLINE, H5E_CANTINIT, FAIL, "unable to add fletcher32 filter to pipeline")
 
     /* Put the I/O pipeline information back into the property list */
-    if(H5P_set(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+    if(H5P_poke(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTSET, FAIL, "can't set pipeline")
 
 done:
@@ -1445,6 +1378,342 @@ H5P_get_filter(const H5Z_filter_info_t *filter, unsigned int *flags/*out*/,
     FUNC_LEAVE_NOAPI(SUCCEED)
 } /* end H5P_get_filter() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:    H5P__ocrt_pipeline_set
+ *
+ * Purpose:     Copies an I/O pipeline property when it's set for a property list
+ *
+ * Return:      Success:        Non-negative
+ *              Failure:        Negative
+ *
+ * Programmer:  Quincey Koziol
+ *              Thursday, Sept 3, 2015
+ *
+ *-------------------------------------------------------------------------
+ */
+static herr_t
+H5P__ocrt_pipeline_set(hid_t H5_ATTR_UNUSED prop_id, const char H5_ATTR_UNUSED *name,
+    size_t H5_ATTR_UNUSED size, void *value)
+{
+    H5O_pline_t     *pline = (H5O_pline_t *)value; /* Create local aliases for values */
+    H5O_pline_t     new_pline;
+    herr_t ret_value = SUCCEED;         /* Return value */
+
+    FUNC_ENTER_STATIC
+
+    /* Sanity check */
+    HDassert(value);
+
+    /* Make copy of I/O pipeline */
+    if(NULL == H5O_msg_copy(H5O_PLINE_ID, pline, &new_pline))
+        HGOTO_ERROR(H5E_PLIST, H5E_CANTCOPY, FAIL, "can't copy I/O pipeline")
+
+    /* Copy new I/O pipeline message over old one */
+    *pline = new_pline;
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5P__ocrt_pipeline_set() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:    H5P__ocrt_pipeline_get
+ *
+ * Purpose:     Copies a layout property when it's retrieved from a property list
+ *
+ * Return:      Success:        Non-negative
+ *              Failure:        Negative
+ *
+ * Programmer:  Quincey Koziol
+ *              Tuesday, Sept 1, 2015
+ *
+ *-------------------------------------------------------------------------
+ */
+static herr_t
+H5P__ocrt_pipeline_get(hid_t H5_ATTR_UNUSED prop_id, const char H5_ATTR_UNUSED *name,
+    size_t H5_ATTR_UNUSED size, void *value)
+{
+    H5O_pline_t     *pline = (H5O_pline_t *)value; /* Create local aliases for values */
+    H5O_pline_t     new_pline;
+    herr_t ret_value = SUCCEED;         /* Return value */
+
+    FUNC_ENTER_STATIC
+
+    /* Sanity check */
+    HDassert(value);
+
+    /* Make copy of I/O pipeline */
+    if(NULL == H5O_msg_copy(H5O_PLINE_ID, pline, &new_pline))
+        HGOTO_ERROR(H5E_PLIST, H5E_CANTCOPY, FAIL, "can't copy I/O pipeline")
+
+    /* Copy new I/O pipeline message over old one */
+    *pline = new_pline;
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5P__ocrt_pipeline_get() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:       H5P__ocrt_pipeline_enc
+ *
+ * Purpose:        Callback routine which is called whenever the pipeline
+ *                 property in the dataset access property list is
+ *                 decoded.
+ *
+ * Return:	   Success:	Non-negative
+ *		   Failure:	Negative
+ *
+ * Programmer:     Mohamad Chaarawi
+ *                 Monday, October 10, 2011
+ *
+ *-------------------------------------------------------------------------
+ */
+static herr_t
+H5P__ocrt_pipeline_enc(const void *value, void **_pp, size_t *size)
+{
+    const H5O_pline_t *pline = (const H5O_pline_t *)value;
+    uint8_t **pp = (uint8_t **)_pp;
+    size_t u;           /* Local index variable */
+
+    FUNC_ENTER_STATIC_NOERR
+
+    HDassert(pline);
+    HDassert(size);
+    HDcompile_assert(sizeof(size_t) <= sizeof(uint64_t));
+
+    if(NULL != *pp) {
+        unsigned enc_size;
+        uint64_t enc_value;
+
+        /* Encode size of unsigned */
+        *(*pp)++ = (uint8_t)sizeof(unsigned);
+
+        /* encode nused value */
+        enc_value = (uint64_t)pline->nused;
+        enc_size = H5VM_limit_enc_size(enc_value);
+        HDassert(enc_size < 256);
+        *(*pp)++ = (uint8_t)enc_size;
+        UINT64ENCODE_VAR(*pp, enc_value, enc_size);
+
+        /* encode each pipeline */
+        for(u = 0; u < pline->nused; u++) {
+            unsigned v;         /* Local index variable */
+
+            /* encode filter ID */
+            INT32ENCODE(*pp, pline->filter[u].id)
+
+            /* encode filter flags */
+            H5_ENCODE_UNSIGNED(*pp, pline->filter[u].flags)
+
+            /* encode filter name if it exists */
+            if(NULL != pline->filter[u].name) {
+                /* encode TRUE indicating that it exits */
+                *(*pp)++ = (uint8_t)TRUE;
+
+                /* encode filter name */
+                HDmemcpy(*pp, (uint8_t *)(pline->filter[u].name), H5Z_COMMON_NAME_LEN);
+                *pp += H5Z_COMMON_NAME_LEN;
+            } /* end if */
+            else
+                /* encode FALSE indicating that it does not exist */
+                *(*pp)++ = (uint8_t)FALSE;
+
+            /* encode cd_nelmts */
+            enc_value = (uint64_t)pline->filter[u].cd_nelmts;
+            enc_size = H5VM_limit_enc_size(enc_value);
+            HDassert(enc_size < 256);
+            *(*pp)++ = (uint8_t)enc_size;
+            UINT64ENCODE_VAR(*pp, enc_value, enc_size);
+
+            /* encode all values */
+            for(v = 0; v < pline->filter[u].cd_nelmts; v++)
+                H5_ENCODE_UNSIGNED(*pp, pline->filter[u].cd_values[v])
+        } /* end for */
+    } /* end if */
+
+    /* calculate size required for encoding */
+    *size += 1;
+    *size += (1 + H5VM_limit_enc_size((uint64_t)pline->nused));
+    for(u = 0; u < pline->nused; u++) {
+        *size += (sizeof(int32_t) + sizeof(unsigned) + 1);
+        if(NULL != pline->filter[u].name)
+            *size += H5Z_COMMON_NAME_LEN;
+        *size += (1 + H5VM_limit_enc_size((uint64_t)pline->filter[u].cd_nelmts));
+        *size += pline->filter[u].cd_nelmts * sizeof(unsigned);
+    } /* end for */
+
+    FUNC_LEAVE_NOAPI(SUCCEED)
+} /* end H5P__ocrt_pipeline_enc() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:       H5P__ocrt_pipeline_dec
+ *
+ * Purpose:        Callback routine which is called whenever the pipeline
+ *                 property in the dataset access property list is
+ *                 decoded.
+ *
+ * Return:	   Success:	Non-negative
+ *		   Failure:	Negative
+ *
+ * Programmer:     Mohamad Chaarawi
+ *                 Monday, October 10, 2011
+ *
+ *-------------------------------------------------------------------------
+ */
+static herr_t 
+H5P__ocrt_pipeline_dec(const void **_pp, void *_value)
+{
+    H5O_pline_t *pline = (H5O_pline_t *)_value;   /* Property to set */
+    const uint8_t **pp = (const uint8_t **)_pp;
+    size_t nused;                       /* Number of filters used for pipeline */
+    unsigned enc_size;                  /* Size of encoded value (in bytes) */
+    uint64_t enc_value;                 /* Value to encode */
+    size_t u;                           /* Local index variable */
+    herr_t ret_value = SUCCEED;         /* Return value */
+
+    FUNC_ENTER_STATIC
+
+    HDcompile_assert(sizeof(size_t) <= sizeof(uint64_t));
+
+    /* Decode the size of size_t */
+    enc_size = *(*pp)++;
+    if(enc_size != sizeof(unsigned))
+        HGOTO_ERROR(H5E_PLIST, H5E_BADVALUE, FAIL, "unsigned value can't be decoded")
+
+    /* decode nused */
+    enc_size = *(*pp)++;
+    HDassert(enc_size < 256);
+    UINT64DECODE_VAR(*pp, enc_value, enc_size);
+    nused = (size_t)enc_value;
+
+    /* Set property default value */
+    HDmemset(pline, 0, sizeof(H5O_pline_t));
+    *pline = H5O_def_pline_g;
+
+    for(u = 0; u < nused; u++) {
+        H5Z_filter_info_t filter;       /* Filter info, for pipeline */
+        uint8_t has_name;               /* Flag to indicate whether filter has a name */
+        unsigned v;                     /* Local index variable */
+
+        /* decode filter id */
+        INT32DECODE(*pp, filter.id)
+
+        /* decode filter flags */
+        H5_DECODE_UNSIGNED(*pp, filter.flags)
+
+        /* decode value indicating if the name is encoded */
+        has_name = *(*pp)++;
+        if(has_name) {
+            /* decode name */
+            filter.name = H5MM_xstrdup((const char *)(*pp));
+            *pp += H5Z_COMMON_NAME_LEN;
+        } /* end if */
+        else
+            filter.name = NULL;
+
+        /* decode num elements */
+        enc_size = *(*pp)++;
+        HDassert(enc_size < 256);
+        UINT64DECODE_VAR(*pp, enc_value, enc_size);
+        filter.cd_nelmts = (size_t)enc_value;
+
+        if(filter.cd_nelmts) {
+            if(NULL == (filter.cd_values = (unsigned *)H5MM_malloc(sizeof(unsigned) * filter.cd_nelmts)))
+                HGOTO_ERROR(H5E_PLIST, H5E_CANTALLOC, FAIL, "memory allocation failed for cd_values")
+        } /* end if */
+        else
+            filter.cd_values = NULL;
+
+        /* decode values */
+        for(v = 0; v < filter.cd_nelmts; v++)
+            H5_DECODE_UNSIGNED(*pp, filter.cd_values[v])
+
+        /* Add the filter to the I/O pipeline */
+        if(H5Z_append(pline, filter.id, filter.flags, filter.cd_nelmts, filter.cd_values) < 0)
+            HGOTO_ERROR(H5E_PLINE, H5E_CANTINIT, FAIL, "unable to add filter to pipeline")
+
+        /* Free cd_values, if it was allocated */
+        filter.cd_values = (unsigned *)H5MM_xfree(filter.cd_values);
+    } /* end for */
+
+done: 
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* H5P__ocrt_pipeline_dec() */
+
+
+/*-------------------------------------------------------------------------
+ * Function:    H5P__ocrt_pipeline_del
+ *
+ * Purpose:     Frees memory used to store the I/O pipeline property
+ *
+ * Return:      Success:        Non-negative
+ *              Failure:        Negative
+ *
+ * Programmer:  Quincey Koziol
+ *              Thursday, Sept 3, 2015
+ *
+ *-------------------------------------------------------------------------
+ */
+static herr_t
+H5P__ocrt_pipeline_del(hid_t H5_ATTR_UNUSED prop_id, const char H5_ATTR_UNUSED *name,
+    size_t H5_ATTR_UNUSED size, void *value)
+{
+    herr_t ret_value = SUCCEED;         /* Return value */
+
+    FUNC_ENTER_STATIC
+
+    /* Sanity check */
+    HDassert(value);
+
+    /* Reset the old I/O pipeline */
+    if(H5O_msg_reset(H5O_PLINE_ID, value) < 0)
+        HGOTO_ERROR(H5E_PLIST, H5E_CANTRESET, FAIL, "can't release I/O pipeline message")
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5P__ocrt_pipeline_del() */
+
+
+/*--------------------------------------------------------------------------
+ * Function:    H5P__ocrt_pipeline_copy
+ *
+ * Purpose:     Copy the I/O pipeline property
+ *
+ * Return:      Success:        Non-negative
+ *              Failure:        Negative
+ *
+ * Programmer:  Quincey Koziol
+ *              Thursday, Sept 3, 2015
+ *
+ *--------------------------------------------------------------------------
+ */
+static herr_t
+H5P__ocrt_pipeline_copy(const char H5_ATTR_UNUSED *name, size_t H5_ATTR_UNUSED size,
+    void *value)
+{
+    H5O_pline_t     *pline = (H5O_pline_t *)value; /* Create local aliases for values */
+    H5O_pline_t     new_pline;
+    herr_t          ret_value = SUCCEED;
+
+    FUNC_ENTER_STATIC
+
+    /* Sanity check */
+    HDassert(pline);
+
+    /* Make copy of I/O pipeline */
+    if(NULL == H5O_msg_copy(H5O_PLINE_ID, pline, &new_pline))
+        HGOTO_ERROR(H5E_PLIST, H5E_CANTCOPY, FAIL, "can't copy I/O pipeline")
+
+    /* Copy new I/O pipeline message over old one */
+    *pline = new_pline;
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5P__ocrt_pipeline_copy() */
+
 
 /*-------------------------------------------------------------------------
  * Function:       H5P__ocrt_pipeline_cmp
@@ -1462,7 +1731,7 @@ H5P_get_filter(const H5Z_filter_info_t *filter, unsigned int *flags/*out*/,
  *-------------------------------------------------------------------------
  */
 static int
-H5P__ocrt_pipeline_cmp(const void *_pline1, const void *_pline2, size_t UNUSED size)
+H5P__ocrt_pipeline_cmp(const void *_pline1, const void *_pline2, size_t H5_ATTR_UNUSED size)
 {
     const H5O_pline_t *pline1 = (const H5O_pline_t *)_pline1,     /* Create local aliases for values */
         *pline2 = (const H5O_pline_t *)_pline2;
@@ -1527,6 +1796,39 @@ done:
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5P__ocrt_pipeline_cmp() */
 
+
+/*-------------------------------------------------------------------------
+ * Function:    H5P__ocrt_pipeline_close
+ *
+ * Purpose:     Frees memory used to store the I/O pipeline property
+ *
+ * Return:      Success:        Non-negative
+ *              Failure:        Negative
+ *
+ * Programmer:  Quincey Koziol
+ *              Thursday, Sept 3, 2015
+ *
+ *-------------------------------------------------------------------------
+ */
+static herr_t
+H5P__ocrt_pipeline_close(const char H5_ATTR_UNUSED *name, size_t H5_ATTR_UNUSED size,
+    void *value)
+{
+    herr_t ret_value = SUCCEED;         /* Return value */
+
+    FUNC_ENTER_STATIC
+
+    /* Sanity check */
+    HDassert(value);
+
+    /* Reset the old I/O pipeline */
+    if(H5O_msg_reset(H5O_PLINE_ID, value) < 0)
+        HGOTO_ERROR(H5E_PLIST, H5E_CANTRESET, FAIL, "can't release I/O pipeline message")
+
+done:
+    FUNC_LEAVE_NOAPI(ret_value)
+} /* end H5P__ocrt_pipeline_close() */
+
 #ifndef H5_NO_DEPRECATED_SYMBOLS
 
 /*-------------------------------------------------------------------------
@@ -1592,7 +1894,7 @@ H5Pget_filter1(hid_t plist_id, unsigned idx, unsigned int *flags/*out*/,
         HGOTO_ERROR(H5E_ATOM, H5E_BADATOM, H5Z_FILTER_ERROR, "can't find object for ID")
 
     /* Get pipeline info */
-    if(H5P_get(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
+    if(H5P_peek(plist, H5O_CRT_PIPELINE_NAME, &pline) < 0)
         HGOTO_ERROR(H5E_PLIST, H5E_CANTGET, H5Z_FILTER_ERROR, "can't get pipeline")
 
     /* Check more args */
